/*
this code is for a traffic light. 4 way intersection. this program does not use delay for timing,
so it can be used in larger projects.
created by: Brett from STEM Mayhem
business: https://www.stemmayhem.com
more info: https://www.stemmayhem.com/arduino-controlled-traffic-lights-for-kids/
*/

// green light minimum time, potentiometer set to minimum
int minGreenDelay = 5000;
// green light max time, potentiometer set to maximum
int maxGreenDelay = 10000;
// orange light wait time
int orangeDelay = 3000;
// time that both directions are red, before green starts
int redDelay = 1000;

// name pins for lights
int r1Pin = 2;
int o1Pin = 3;
int g1Pin = 4;
int r2Pin = 6;
int o2Pin = 7;
int g2Pin = 8;
// name the pin for potentiometer/speed input
int speedPin = A0;


/*
no need to change anything below here
*/


// set initial times for lights
unsigned long greenMillis = 0;
unsigned long orangeMillis = 0;
unsigned long redMillis = 0;
// set current direction
int currentDirection = 1;
// set the starting status - we set it green
bool greenStatus = true;
bool orangeStatus = false;
bool redStatus = false;
  
// start the setup
void setup() {
  // initialise pins for leds as outputs
  pinMode(r1Pin, OUTPUT);
  pinMode(o1Pin, OUTPUT);
  pinMode(g1Pin, OUTPUT);
  pinMode(r2Pin, OUTPUT);
  pinMode(o2Pin, OUTPUT);
  pinMode(g2Pin, OUTPUT);
  // turn on green light direction 1
  digitalWrite(g1Pin, HIGH);
  // turn on red light direction 2
  digitalWrite(r2Pin, HIGH);
}




// start the loop
void loop() {
  // read the potentiometer for speed
  int sensorValue = analogRead(speedPin);
  // delay 1 millisecond for reading/saving time
  delay(1);
  // calculate the delay using mapping
  int greenDelay = map(sensorValue, 0, 1023, minGreenDelay, maxGreenDelay);
  // store current time
  unsigned long currentMillis = millis();
  
  // if green time is reached and green is the current mode
  if (currentMillis - greenMillis >= greenDelay && greenStatus == true) {
    // set orange millis start time to current time
    orangeMillis = millis();
    // change to orange using function
    goOrange(currentDirection);
    // delay 4 milliseconds because of millis bug
    delay(4);
    // un set the green mode
    greenStatus = false;
    // set the current mode to orange
    orangeStatus = true;
  }
  
  // if orange time is reached and orange is the current mode
  if (currentMillis - orangeMillis >= orangeDelay && orangeStatus == true) {
    // set red millis start time to current time
    redMillis = millis();
    // change to orange using function
    goRed(currentDirection);
    // un set the orange mode
    orangeStatus = false;
    // set the current mode to red
    redStatus = true;
  }
  
  // if red is on (both ways) then check the time and change to green other direction
  if (currentMillis - redMillis >= redDelay && redStatus == true) {
    // set green millis start time to current time
    greenMillis = millis();
    // change to green using function
    goGreen(currentDirection);
    redStatus = false;
    // unset the red mode
    greenStatus = true;
    // change direction to the new direction
   currentDirection = (currentDirection == 2) ? 1: 2; 
  }
}


// function to change to orange just needs current direction
void goOrange(int currentDirection){
   if(currentDirection == 1)
  {
  digitalWrite(g1Pin, LOW);
  digitalWrite(o1Pin, HIGH);
  }
  else
  {
  digitalWrite(g2Pin, LOW);
  digitalWrite(o2Pin, HIGH);
  }
}

// function to change to red just needs current direction
void goRed(int currentDirection){
   if(currentDirection == 1)
  {
  digitalWrite(o1Pin, LOW);
  digitalWrite(r1Pin, HIGH);
  }
  else
  {
  digitalWrite(o2Pin, LOW);
  digitalWrite(r2Pin, HIGH);
  }
}

// function to change to green just needs current direction
void goGreen(int currentDirection){
   if(currentDirection == 1)
  {
  digitalWrite(r2Pin, LOW);
  digitalWrite(g2Pin, HIGH);
  }
  else
  {
  digitalWrite(r1Pin, LOW);
  digitalWrite(g1Pin, HIGH);
  }
}







